Effects
=======

{{toc}}

Introduction
------------

Effects are a special kind of action. Instead of modifying normal properties like opacity, position, rotation, or scale, they modify a new kind of property: the grid property.

A grid property is like a matrix. It is a network of lines that cross each other to form a series of squares or rectangles.

These special actions render any cocos2d-x Node object (Layer, Scene, Sprite, etc.) into the grid and you can transform the grid by moving its vertices.

There are 2 kind of grids: tiled grids and non-tiled grids. The difference is that the tiled grid is composed of individual tiles while the non-tiled grid is composed of vertices.

![](http://www.cocos2d-x.org/attachments/1575/tiled_and_nontiled_grid.png)

The following is an example of Ripple3D action, who uses a Grid3D (non-tiled) grid:
![](http://www.cocos2d-x.org/attachments/1576/effect_ripple3d.png) ![](http://www.cocos2d-x.org/attachments/1577/effect_ripple3d_grid.png)

and the following is an example of FadeOutTR action, who uses a TiledGrid3D (tiled) grid:
![](http://www.cocos2d-x.org/attachments/1578/effect_fadeouttiles.png) ![](http://www.cocos2d-x.org/attachments/1579/effect_fadeouttiles_grid.png)

How do I use them ?
-------------------

Like any other action, it is executed by the `runAction`. eg:

````cpp
    // Create an sprite
    Sprite* grossini = Sprite::create("grossini.png");
    grossini->runAction(Liquid::create(2, Size(32,24), 1, 20));
````

````lua
    -- Create an sprite
    local grossini = cc.Sprite:create("grossini.png")
    grossini:runAction(cc.Liquid:create(2, cc.size(32,24), 4, 20))
````

Liquid, like any other grid action, receives the grid parameter. You can adjust the quality of the effect by increasing the size of the grid. But it also implies less FPS.

The Effects are `IntervalAction` actions so you can treat them like any other action. eg:
````cpp
    // create a Lens3D action
    ActionInterval* lens = Lens3D::create(10, Size(32,24), Vec2(100,180), 150)?

    // create a Waved3D action
    ActionInterval* waves = Waves3D::create(10, Size(15,10), 18, 15);

    // create a sequence an repeat it forever
    grossini->runAction(RepeatForever::create((Sequence*)Sequence::create(waves, lens, NULL ) ) );
````

````lua
    -- create a Lens3D action
   local lens = cc.Lens3D:create(10, cc.size(32,24), cc.p(100,180), 150)

    -- create a Waved3D action
   local waves = cc.Waves3D:create(10, cc.size(15,10), 18, 15)

    --create a sequence an repeat it forever
    grossini:runAction(cc.RepeatForever:create(cc.Sequence:create(waves, lens)))
````
The following are Grid3D (non-tiled) actions in v2.1.0:

|Shaky3D|Waves3D|FlipX3D|FlipY3D|
|-------|-------|-------|-------|
|![](http://www.cocos2d-x.org/attachments/1580/IMG_0001.PNG)|![](http://www.cocos2d-x.org/attachments/1581/IMG_0002.PNG)|![](http://www.cocos2d-x.org/attachments/1582/IMG_0003.PNG)|![](http://www.cocos2d-x.org/attachments/1583/IMG_0004.PNG)|

|Lens3D|Liquid|Waves|Twirl|
|------|------|-----|-----|
|![](http://www.cocos2d-x.org/attachments/1584/IMG_0005.PNG)|![](http://www.cocos2d-x.org/attachments/1585/IMG_0007.PNG)|![](http://www.cocos2d-x.org/attachments/1586/IMG_0008.PNG)|![](http://www.cocos2d-x.org/attachments/1587/IMG_0011.PNG)|

|Ripple3D|
|--------|
|![](http://www.cocos2d-x.org/attachments/1588/IMG_0002_2.PNG)|

The following are TiledGrid3D (tiled) actions:

|ShakyTiles3D|ShatteredTiles3D|ShuffleTiles|FadeOutTRTiles|
|------------|----------------|------------|--------------|
|![](http://www.cocos2d-x.org/attachments/1593/IMG_0012.PNG)|![](http://www.cocos2d-x.org/attachments/1594/IMG_0013.PNG)|![](http://www.cocos2d-x.org/attachments/1595/IMG_0014.PNG)|![](http://www.cocos2d-x.org/attachments/1596/IMG_0015.PNG)|

|FadeOutBLTiles|FadeOutUpTiles|FadeOutDownTiles|TurnOffTiles|
|--------------|--------------|----------------|------------|
|![](http://www.cocos2d-x.org/attachments/1597/IMG_0016.PNG)|![](http://www.cocos2d-x.org/attachments/1598/IMG_0017.PNG)|![](http://www.cocos2d-x.org/attachments/1599/IMG_0019.PNG)|![](http://www.cocos2d-x.org/attachments/1600/IMG_0020.PNG)|

|WavesTiles3D|JumpTiles3D|SplitRows|SplitCols|
|------------|-----------|---------|---------|
|![](http://www.cocos2d-x.org/attachments/1601/IMG_0021.PNG)|![](http://www.cocos2d-x.org/attachments/1602/IMG_0022.PNG)|![](http://www.cocos2d-x.org/attachments/1603/IMG_0023.PNG)|![](http://www.cocos2d-x.org/attachments/1604/IMG_0024.PNG)|

References
----------

cocos2d-iphone [Introduction to cocos2ds effects](http://www.cocos2d-iphone.org/archives/40)