Facebook integration instruction for Cocos2d-x and Cocos2d-x-Lua on iOS
======================================================================
This doc walks you through the usage of Facebook Integration for Cocos2d-x and Cocos2d-x-Lua on iOS.

## Create a Facebook App

Please refer to [Step 3. Obtain a Facebook App ID - Getting Started with the Facebook iOS SDK](http://developers.facebook.com/docs/ios/getting-started/#appid) to create a Facebook App on iOS platform.

## Integrate Facebook into your cpp iOS Project
**Notice**:Our project was created by `cocos new` which is the Cocos command.

**Step1**: Open the your Xcode project file.Assuming the project name "MyProject"

**Step2**: Right click `MyProject`,select `Add Files to "MyProject"` then add the `MyProject/cocos2d/plugin/plugins/facebook/proj.ios/PluginFacebook.xcodeproject` and `MyProject/cocos2d/plugin/protocols/proj.ios/PluginProtocol.xcodeproj` ?
![](http://www.cocos2d-x.org/attachments/download/5375)

**Step3**: Click on the project root, then choose `target`->`MyProject iOS`->`Build Phases`->`Link Binary With Libraries`. Click `+` button to popup a sub window. Then click on `Add Other...` button in the sub window. Then find `FacebookSDK.framework` under `MyProject/cocos2d/plugin/plugins/facebook/proj.ios/sdk/` to add it.
![](http://www.cocos2d-x.org/attachments/download/5376)

**Step4** Click target `MyProject iOS`,then select `Build Phases` and click `+` of `Target Dependencies` to add `libPluginProtocol.a` and `libPluginFacebook.a`

![](http://www.cocos2d-x.org/attachments/download/5379)

**Step5**: Refer to [Step 4: Configure your Xcode Project - Getting Started with the Facebook iOS SDK](http://developers.facebook.com/docs/ios/getting-started/#configure) to add `FacebookAppID`, `FacebookDisplayName`, `URL types` and check the `Bundle identifier` in `Info.plist` file. You can find this file under `ios` folder of your Xcode project. You can see the result below:
![](http://www.cocos2d-x.org/attachments/download/5377)

**Step6**: Choose target `MyProject iOS` in the project root, open `Build Settings` page, then search for `Other Linker Flags`, and add `-ObjC` linker flag:
![](http://www.cocos2d-x.org/attachments/download/5380)

**Step7**:Find `ios/AppController.mm`, add necessary codes for Facebook iOS SDK:

- Add header:

	```

	 #import &ltFacebookSDK/FacebookSDK.h&gt

	```

- Add the following method in the `@implementation` section: 

	```
	- (BOOL)application:(UIApplication *)application openURL:(NSURL *)url sourceApplication:(NSString *)sourceApplication annotation:(id)annotation
	{
	    return [FBSession.activeSession handleOpenURL:url];
	}
	```

- Insert `[FBAppCall handleDidBecomeActive];` in `applicationDidBecomeActive:` function: 

	```
	- (void)applicationDidBecomeActive:(UIApplication *)application {
	    [FBAppCall handleDidBecomeActive];
	    cocos2d::Director::getInstance()->resume();
	}
	```

Through these steps,we could begin to use the Facebook API.

## Integrate Facebook into your Lua iOS Project
The most steps are similar with ` Integrate Facebook into your cpp iOS Project`,the special steps as follows:
**Step1**: Add related lua bindins files to the project

![](http://www.cocos2d-x.org/attachments/download/5381)

**Step2**:  Add necessary codes

- Add related header files of Lua bindings in `AppDelegate.cpp`
```
        #include "lua_cocos2dx_pluginx_auto.hpp"
        #include "lua_pluginx_manual_callback.h"
        #include "lua_pluginx_manual_protocols.h"
```

- Register lua bindings codes in `AppDelegate::applicationDidFinishLaunching`function of `AppDelegate.cpp`.
**Notice**: Please pay attention to the location of the codes.
    
```
    bool AppDelegate::applicationDidFinishLaunching()
    {
        ...

        auto engine = LuaEngine::getInstance();
        ScriptEngineManager::getInstance()->setScriptEngine(engine);

        ...

        //register custom function
        auto luaState = stack->getLuaState();
        lua_getglobal(luaState, "_G");
        register_all_pluginx_protocols(luaState);
        register_all_pluginx_manual_callback(luaState);
        register_all_pluginx_manual_protocols(luaState);

        sc->start();    
        
        ...

        engine->executeScriptFile(ConfigParser::getInstance()->getEntryFile().c_str());
        return true;
    }
  ```

**Step3**:  Select `TARGETS`->`Build Phases`,then click `+` to  add `New Run Script Build Phase` to copy some basic script.The related setting in the project is as follow:

![](http://www.cocos2d-x.org/attachments/download/5382)

Through these special steps,we could begin to use the Facebook API by Lua.
 
To know more detail about Plugin-x,please refer to [Plugin-x Architecture](http://www.cocos2d-x.org/docs/manual/framework/html5/jsb/plugin-x/plugin-x-architecture/en)

## How to Use Facebook API
Please visit [Facebook API Reference for Cocos2d-x and Cocos2d-x-lua](http://www.cocos2d-x.org/wiki/Facebook_API_Reference_for_Cocos2d-x_and_Cocos2d-x-lua)