# How To Create A HUD

## What is a HUD?

A HUD, or "Heads Up User Display"  is a way of displaying data to the user without them having to look away from what the are doing. HUDS are almost certainly used in most every game. Here are some examples of HUDS is game we all know:

![m_kfm](m_kfm.png)

![m_pitfall](m_pitfall.png)

![m_excite](m_excite.jpg)

![m_mario](m_mario.png)

## What information goes in a HUD?

Anything! Score, Health, Messages, Achievements, Power-Ups, Progress indicators, Timers

## How do you make a HUD?

Think of a HUD as a type of `Layer*`, more specifically in Cocos2d-x a type of `LayerColor*`. [API](http://www.cocos2d-x.org/reference/native-cpp/V3.0/d1/de8/classcocos2d_1_1_layer_color.html#ab130d18f75bae5aab246f43f87a00034)   

Lets create a simple HUD that when called tells the user a message.

header:
````cpp
#ifndef  _MESSAGEHUD_HPP_
#define  _MESSAGEHUD_HPP_

#include "cocos2d.h"

class MessageHUD : public cocos2d::LayerColor
{
    public:
        MessageHUD();
        ~MessageHUD();
    
        MessageHUD* createLayer(const std::string& _message);
        void initOptions(const std::string& _message);
    
        virtual void draw (cocos2d::Renderer * renderer, const cocos2d::Mat4 & transform,
                         bool transformUpdated); 
    
        inline virtual cocos2d::LabelBMFont* getMessageLabel() { return _messageLabel; };
    
    private:
        cocos2d::LabelBMFont* _messageLabel;
};

#endif // _MESSAGEHUD_HPP_
````

source:
````cpp

#include "MessageHUD.hpp"

MessageHUD::MessageHUD() {}

MessageHUD::~MessageHUD() {}

MessageHUD* MessageHUD::createLayer(const std::string& _message)
{   
    MessageHUD* a = new MessageHUD();
    a->create();
    a->setColor(cocos2d::Color3B(0, 0, 0));
    
    a->setContentSize(cocos2d::Size::Size(1, 1));
    a->setAnchorPoint(cocos2d::Vec2(0, 0)); 

    a->initOptions(_message);
    
    return a;
}

void MessageHUD::initOptions(const std::string& _message)
{
    _messageLabel = cocos2d::LabelBMFont::create(_message.c_str(), "Marker Felt Small.fnt");
    _messageLabel->setColor(cocos2d::Color3B(255, 215, 0));
    
    addChild(_messageLabel, 1);
    
    _messageLabel->setPosition(cocos2d::Vec2(Director::getInstance()->getVisibleSize().width, 
        Director::getInstance()->getVisibleSize().height));
 

}

void MessageHUD::draw(cocos2d::Renderer * renderer, const cocos2d::Mat4 & transform,
                         bool transformUpdated) {}
````

to create:
````cpp
MessageHUD* _hud = MessageHUD::createLayer("The Enemy is approaching!");
this->addChild(_hud, 2);
````