# How To Create Sprites

`Sprites` are a core element to any game. We need them. We often come to love the characters in games we make and play.

There are many ways to create `Sprites`  [API](http://www.cocos2d-x.org/reference/native-cpp/V3.0/d8/de9/group__sprite__nodes.html#ga43791ab945bb6a98ee0ee9dc006bc253) 

````cpp
cocos2d::Sprite* _mySprite = cocos2d::Sprite::create(); // empty sprite with no texture
````

````cpp
cocos2d::Sprite* _mySprite = cocos2d::Sprite::create("mysprite.png"); 
````

 What happens when you have a lot of sprites and you need to keep track of them? We need to use a data structure for this. What are our options? `Array` [API](http://www.cocos2d-x.org/reference/native-cpp/V3.0/df/db3/group__data__structures.html#gabfc3cc657372e669cb9abc09de84eedc)  or `Vector` [API](http://www.cocos2d-x.org/reference/native-cpp/V3.0/d4/d38/classcocos2d_1_1_vector.html)  perhaps? If your needs are simple, the C++ STL also has `array` and `vector` containers.
````cpp
std::vector<Sprite*> _vMySprites;
_vMySprites.reserve(5); // this may or may not make sense for your usage

for (int i=0;i < _vMySprites.capacity(); i++)
{
    cocos2d::Sprite* foo = cocos2d::Sprite::create("mysprite.png");
    
    int random = rand() % 5;

    foo->setPosition(cocos2d::Point((60 * random), (50 * random)));
    foo->setTag(i);

    _vMySprites.push_back(foo); // add our sprite to vector for storage.

    addChild(foo, 1);
}
````
