How to Debug Cocos2d-JS Game Using Code IDE
==============================================
---
Supportive Debugging Environment


On Windows you can debug Windows and Android game, and on Mac you can debug Mac, iOS and Android game.

This manual makes use of Mac as development environment, and here we will show you how to use Code IDE. The process on Windows is essentially the same.

{{toc}}

Prerequisite
===
---
Please refer to [Cocos Code IDE][Cocos Code IDE] if you have not completed the installation of Cocos Code IDE. 


IDE Configuration 
===
---

## Basic Settings

In **Preferences/Cocos/JavaScript** page, select the directory of Cocos2d-JS 3.x you had downloaded to be Javascript Framework. 
+ ![](http://www.cocos2d-x.org/attachments/download/5279)

## Additional settings

You should provide compiling tools in Android if you need to replace the Android runtime we provided. Open Preferences / Cocos page. 
+ ![](http://cocos2d-x.org/attachments/download/5262)

Debug the Game
===
---
1. Create a new Cocos2d-JS project and name it CocosJSGame
2. Click Debug button on toolbar
+ ![](http://www.cocos2d-x.org/attachments/download/5200)
3. The script will be running on Runtime of Mac version we provided by default. If you want to debug program on other target platforms we provided, please refer to [How To Debug on Other Target Platforms][How To Debug on Other Target Platforms].

How to Debug
===
---

## Breakpoints Supporting 
You are able to create breakpoints in JavaScript script files.
+ When the close button is clicked, the breakpoint will be triggered.
+ When the breakpoint is triggered, select "Yes" to open **Debug Perspective**, and you will find many useful debug views, Call stacks, Variables, Breakpoints, etc.
+ ![](http://www.cocos2d-x.org/attachments/download/5205)
+ **Step over, Step into, Step out are also supported**


## Code Hot Updating 
Do you want to change the position of close button?
The closeItem.attr function, is controlling the close menu's position.


```javascript
		closeItem.attr({
            x: size.width - 20,
            y: 20,
            anchorX: 0.5,
            anchorY: 0.5
        });
```


Change "size.width-20" to ""size.width/2" on "x" and save your change, then you will find that you have changed where the close button is without restarting the app! It is now at the middle of the bottom! 


## How to Debug on Other Target Platforms
---
+ Open Debug Configurationspage by clicking **Debug Configurations** button on toolbars
+ ![](http://www.cocos2d-x.org/attachments/download/5207)
+ On the left side of the page, select the configuration named CocosJSGame we created just now.

### Debug on iOS Simulator
+ Check **iOS Simulator** radio button
+ Choose a runtime app
+ ![](http://www.cocos2d-x.org/attachments/download/5280)
+ Then click the **Debug** button
+ IDE will auto-install chosen runtime app and start runtime to debug

### Debug on iOS Device
+ Firstly, you need a runtime IPA, from Cocos2d-JS 3.0 rc2 version offers PrebuiltRuntimeJs.ipa, from 1.0.0-rc1 version, you can build a custom runtime IPA by Cocos Code IDE too.  
+ [Install runtime IPA][How To Install IPA] to your iOS device by iTunes and start runtime manually.
+ On "Debug Configuration" page, Check **Remote Debug** radio button 
+ Select "iOS"" platform
+ Enter IP address of your device into the **Target IP**(Make sure that the **Target IP** is accessible)
+ ![](http://www.cocos2d-x.org/attachments/download/5275)
+ Then click the **Debug** button

### Debug on Android Device by ADB Mode
+ Check **Android ADB Mode** radio button
+ Choose a runtime apk
+ ![](http://www.cocos2d-x.org/attachments/download/5278)
+ Click the **Debug** button
+ IDE will auto-install chosen runtime apk and start runtime to debug

### Debug on Android Device by WLAN
+ Manually install runtime apk to your device. It is placed in CocosJSGame/runtime/android.
+ Start the runtime on device manually
+ On "Debug Configuration" page, Check **Remote Debug** radio button
+ Select "Android"" platform
+ Enter IP address of your device into the **Target IP**(Make sure that the **Target IP** is accessible)
+ ![](http://www.cocos2d-x.org/attachments/download/5281)
+ Click the **Debug** button

Enjoy!

[Cocos Code IDE]: Cocos_Code_IDE
[How To Install IPA]:http://www.solutionanalysts.com/blog/how-install-ipa-file-iphone-ipod-ipad-using-itunes-mac-windows
[How To Debug on Other Target Platforms]: http://cocos2d-x.org/wiki/How_to_Debug_Cocos2d-x_JS_Game_Using_Code_IDE#How-to-Debug-on-Other-Target-Platforms