How to Debug Cocos2d-x Lua Game Using Code IDE
===
---
Supportive Debugging Environment 

On Windows you can debug Windows and Android games, and on Mac you can debug Mac, iOS and Android games.

This manual makes use of Mac as development environment, and here we will show you how to use Code IDE. The process on Windows is essentially the same.

{{toc}}

Prerequisite
===
---
Please refer to [Cocos Code IDE][Cocos Code IDE] if you have not completed the installation of Cocos Code IDE. 

IDE Configuration 
===
---
## Basic Settings
Open **Preferences/Cocos/Lua** page, select the directory of cocos2d-x 3.x you just downloaded to be Lua Framework. 

+ ![](http://www.cocos2d-x.org/attachments/download/5265)

## Additional Settings

You should provide compiling tools in Android if you need to replace the Android runtime we provided. Open Preferences / Cocos page. 

+ ![](http://www.cocos2d-x.org/attachments/download/5262)


Debug the Game
===
---
1. Create a new Cocos Lua project and name it CocosLuaGame. 
2. Click Debug button on toolbar
3. ![](http://www.cocos2d-x.org/attachments/download/5197)
4. The script will be running on Runtime of Mac version we provided by default. If you want to debug program on other target platforms we provided, please refer to [How To Debug on Other Target Platforms][How To Debug on Other Target Platforms].

How to Debug
===
---

## Breakpoints Supporting 
+ You are able to create breakpoints in Lua script files.
+ When the breakpoint is triggered, select "Yes" to open **Debug Perspective**, and you will find many useful debug views, Call stacks, Variables, Breakpoints, etc.
+ ![](http://www.cocos2d-x.org/attachments/download/5194)
+ **Step over, Step into, Step out are also supported**

## Code Hot Updating 
Do you want to change the moving path of SpriteDog?  
The "tick()" function controls the SpriteDog's position.

```Lua
-- moving dog at every frame
local function tick()
    if spriteDog.isPaused then return end
    local x, y = spriteDog:getPosition()
    if x > origin.x + visibleSize.width then
        x = origin.x
    else
        x = x + 1
    end

    spriteDog:setPositionX(x)
end    
```
    
Modify the implementation of function, for example, change the value 1 to 10 and save your change. 
Then you will find that you have improved the speed of SpriteDog without restarting the app!

## How to Debug on Other Target Platforms
---
+ Open Debug Configurationspage by clicking **Debug Configurations** button on toolbars
+ ![](http://www.cocos2d-x.org/attachments/download/5196)
+ On the left side of the page, select the configuration named CocosLuagame we created just now.

### Debug on iOS Simulator
+ Check **iOS Simulator** radio button
+ Choose a runtime app
+ ![](http://www.cocos2d-x.org/attachments/download/5266)
+ Then click the **Debug** button
+ IDE will auto-install chosen runtime app and start runtime to debug

### Debug on iOS Device
+ Firstly, you need a runtime IPA, you can build a custom runtime IPA by Code IDE from version 1.0.0-rc1, then [Install runtime IPA][How To Install IPA] to iOS device.
+ On "Debug Configuration" page, Check **Remote Debug** radio button 
+ Select "iOS" platform
+ Enter IP address of your device into the **Target IP** and select the IP address that your PC used on the **Host IP**(Make sure that the **Target IP** and **Host IP**  can access each other)
+ ![](http://www.cocos2d-x.org/attachments/download/5268)
+ Then click the **Debug** button

### Debug on Android Device by ADB Mode
+ Check **Android ADB Mode** radio button
+ Choose a runtime apk
+ ![](http://www.cocos2d-x.org/attachments/download/5264)
+ Click the **Debug** button
+ IDE will auto-install chosen runtime apk and start runtime to debug

### Debug on Android Device by WLAN
+ Manually install runtime apk to your device. It is placed in CocosLuaGame/runtime/android.
+ Start the runtime on device manually
+ On "Debug Configuration" page, Check **Remote Debug** radio button 
+ Select "Android" platform
+ Enter IP address of your device into the **Target IP** and select the IP address that your PC used on the **Host IP**(Make sure that the **Target IP** and **Host IP**  can access each other)
+ ![](http://www.cocos2d-x.org/attachments/download/5267)
+ Then click the **Debug** button

Enjoy!

[Cocos Code IDE]: Cocos_Code_IDE
[How To Install IPA]:http://www.solutionanalysts.com/blog/how-install-ipa-file-iphone-ipod-ipad-using-itunes-mac-windows
[How To Debug on Other Target Platforms]: http://cocos2d-x.org/wiki/How_to_Debug_Cocos2d-x_Lua_Game_Using_Code_IDE#How-to-Debug-on-Other-Target-Platforms