How to use pthread
==================

Now, you can use pthread in cocos2d-x, but there are some limitations.

1. Dont call any functions which invokes `Ref::retain()`, `Ref::release()` or `Ref::autorelease()`, because AutoreleasePool are not thread-safe. Please refer to [[Reference Count and AutoReleasePool in Cocos2d-x]] for more details. Cocos2d-x use AutoreleasePool every where in its framework, so my suggestion is that, dont invoke any cocos2d-x API in a new thread except [[Data Structures]].

2. If you want to load resources in a new thread, you can use `TextureCache::addImageAsync()`

3. `pthread_cond_wait()` seems have a bug, it can not wait at the first time, but works properly in subsequence.

If we make retain(), release() and autorealese() thread-safe, then mutex would be required. For the reason that cocos2d-x framework releases the autorelease pool at each end of message loop, using mutex may cause performance issue.

By the way, OpenGL context is not thread-safe, which you should always keep in mind.

cocos2d-x/extensions/network/HttpClient.cpp uses `pthread_t` and `pthread_mutex_t` to create network thread. You can look into it as a sample.

Simple pthread example that shows how to run a function in a separate thread, pass a structure to it and set a mutex:

    pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;

    struct SimpleStructure
    {
        int data;
        float otherData;
    };

    void* ThreadFunction(void* arg)
    {
        pthread_mutex_lock(&mutex);
        SimpleStructure* args = (SimpleStructure*)arg;
        // do something with args->data and args->otherData
        delete args;
        pthread_mutex_unlock(&mutex);
        return NULL;
    }

    void CreateThread()
    {
        pthread_t thread;
        SimpleStructure* args = new SimpleStructure();
        args->data = 1;
        args->otherData = 2.0f;
        pthread_create(&thread, NULL, &ThreadFunction, args);
    }
