# Plugin-X Integration Guide for Android

{{toc}}

[[Third_Party_SDK_Integration]]

## Environment Requirement

1.  python 2.7. http://www.python.org/
2.  Apache Ant build tool. http://ant.apache.org/
3.  If your OS is Windows, you should have Cygwin environment.

## Compiling the Plugin-X Project

1. Run the script `publish.sh` which is in the path `plugin-x/tools/publish.sh`. This script has been well tested on both Mac OSX Mountain Lion v10.8.2 and Cygwin on Windows 7. At this scripts first run, it will ask you to input some parameters for setting up environment, like this
![](http://www.cocos2d-x.org/attachments/download/1837)

On Windows, the path you input should be Linux style. For example : You can input `C:/adt-bundle-windows/sdk`, but not `C:\adt-bundle-windows\sdk`

2. After the compilation finishes, a directory named *publish* will be generated in the root directory of the plugin-x projet. If your *publish* directory looks like this, it means that your compilation is successful.

![](http://www.cocos2d-x.org/attachments/download/1992)

3. The files in *publish* directory involves:
* plugin header files (**.h)
* C++ static libs
* java libs
* makefiles for android
* android projects needed by some plugins

## Modify the game project before using plugin

### What we need to do?
Plugin-x wrapper C/C++ interfaces for the third party android SDKs. So we should modify the game project configuration to use it:

* Modify the .mk files(Android.mk & Application.mk) to link the static libs built by plugin-x.
* Modify the android project configuration to link the libs built by plugin-x & third party SDK libs. (.jar files)
* Modify the AndroidManifest.xml, add the declaration of activities, and user permissions for game.
* For some particular plugins, we need to add external configuration and resource files.

### Game developer guide tool

Now we have a script named `gameDevGuide.sh` which is in the path `plugin-x/tools/gameDevGuide.sh`.
It will help you modify your game project.

1. Run the script in terminal (On Windows, you should run it in Cygwin). The UI will be shown like this:

![](http://www.cocos2d-x.org/attachments/download/1830)

Input android project path of your game into the edit-box. Please keep the path have no spaces. Then click Next button.

2. Select the plugins you needed. The UI looks like:

![](http://www.cocos2d-x.org/attachments/download/1831)

After select plugins, click Finish button. The script will modify some configuration files of your android project.
The follow configuration files will be modified:
* Android.mk  Add C++ libs dependence for game
* Application.mk  Add parameter for ndk build
* .project  Link the publish directory to the game project as extern resource
* .classpath  Add java libs dependence for game
* AndroidManifest.xml  Add activity declaration & user permission needed by plugins for game

### Manual modified

1. Modify the ndk-build command parameter : add the *publish* directory into parameter NDK_MODULE_PATH , such as :
```
NDK_MODULE_PATH=${PLUGIN_ROOT}/publish
```
2. Add code into the method `JNI_OnLoad`, like this :

```
#include PluginJniHelper.h
jint JNI_OnLoad
{
 JniHelper::setJavaVM;
 PluginJniHelper::setJavaVM; // for plugins
 return JNI_VERSION_1_4;
}
```

3. Invoke method `PluginWrapper.init()` when the main activity of the game is created :

```
import org.cocos2dx.plugin.PluginWrapper;
import org.cocos2dx.lib.Cocos2dxGLSurfaceView;
public class HelloIAP extends Cocos2dxActivity{
 protected void onCreate{
 super.onCreate();
 PluginWrapper.init(); // for plugins
 // If you want your callback function can be invoked in GL thread, add this line:
 PluginWrapper.setGLSurfaceView();
 }
 static {
 System.loadLibrary;
 }
}
```

4. Particular modifications for special plugin
If your game use the following plugins, you need some particular modifications.
We have plan to add these modifications into the developer guide tool.

* For nd91  
Import the android project `91SDK_LibProject_complete` into eclipse. The path of this project is `plugin-x/publish/nd91/andriod/DependProject`.  
Make your game project depend on the project `91SDK_LibProject_complete`, like this:
![](http://www.cocos2d-x.org/attachments/download/1832)

## Using plugin-x in your C++ code

### load and unload plugin

All plugins are managed by class PluginManager . You can load/unload one plugin by its class name , sample code :
```
    // load plugin AnalyticsFlurry
    s_pFlurry = dynamic_cast
    (PluginManager::getInstance()->loadPlugin("AnalyticsFlurry"));

    // unload plugin AnalyticsFlurry
    PluginManager::getInstance()->unloadPlugin("AnalyticsFlurry");
    s_pFlurry = NULL;
```
Suggestion : Load the plugin when the game begin, and unload it when the game end.

### using plugin

You can directly invoke the methods declared in protocols. Sample code:
```
    // enable the debug mode
    s_pFlurry->setDebugMode(true);

    // log an event
    s_pFlurry->logEvent("music");

    // log an event with params
    LogEventParamMap paramMap;
    paramMap.insert(LogEventParamPair("type", "popular"));
    paramMap.insert(LogEventParamPair("artist", "JJLin"));
    s_pFlurry->logEvent("music", &paramMap);
```
Some plugins have custom methods. We implement a simple reflection in plugin-x.
You can invoke the custom methods by name. Like this:
```
    // invoke setAge() method in plugin Flurry
    PluginParam pParam3(20);
    s_pFlurry->callFuncWithParam("setAge", &pParam3, NULL);

    // invoke logTimedEventBeginWithParams() method in plugin Flurry
    PluginParam event3("music-kv");
    LogEventParamMap paramMap;
    paramMap.insert(LogEventParamPair("type", "popular"));
    paramMap.insert(LogEventParamPair("artist", "JJLin"));
    PluginParam mapValue(paramMap);
    s_pFlurry->callFuncWithParam("logTimedEventBeginWithParams", &event3, &mapValue, NULL);
```
Also, there are some interfaces defined for reflection to return basic-type values. Methods List:
```
    // return string value for developers
    const char* callStringFuncWithParam(const char* funcName, PluginParam* param, ...);

    // return int value for developers
    int callIntFuncWithParam(const char* funcName, PluginParam* param, ...);

    // return bool value for developers
    bool callBoolFuncWithParam(const char* funcName, PluginParam* param, ...);

    // return float value for developers
    float callFloatFuncWithParam(const char* funcName, PluginParam* param, ...);
```
Attention:
* You can pass parameters to methods, the parameters must be pointer of PluginParam.
* The parameters must be passed in the plugin required order.
* If the method cant be found in plugin, nothing will happen.

References
----------

1.  The screenshots and sample codes come from plugin-xs sample project HelloIAP
2.  If you have any question, please dont hesitate to add comments on this wiki page, or create issues on github.com/chukong/plugin-x/ project.

