Reference Count and AutoReleasePool in Cocos2d-x
================================================

> version: Cocos2d-x v3.x
> update: {{lastupdated_at}}

{{toc}}

Reference Count
---------------

Reference count is an old approach of memory management in C/C++ projects. When I dived to [TCPMP](http://en.wikipedia.org/wiki/The_Core_Pocket_Media_Player) open source project 8 years ago, reference count was there.
iOS SDK wrapped this mechanism in `NSAutoreleasePool`. So we have a cloned one `AutoreleasePool` in cocos2d-x. The usage is almost the same, so if you havent developed on iOS, please read Apples official document first [NSAutoreleasePool Class Reference](http://developer.apple.com/library/mac/#documentation/Cocoa/Reference/Foundation/Classes/NSAutoreleasePool_Class/Reference/Reference.html)

And we tried to use [std::shared_ptr](http://en.cppreference.com/w/cpp/memory/shared_ptr) instead, but gave up because of performance issue.

AutoreleasePool
-----------------

`AutoreleasePool` of cocos2d-x has the same concept and the similar APIs with `cocoa NSAutoreleasePool`, but with one significant difference:

* `AutoreleasePool` can not be used in multi-thread. So if your game needs a network thread, please just receive the data and change state flags in the network thread, dont call cocos2d interfaces in it. The reason is explained below.

The logic of `AutoreleasePool` is that, when you call `object->autorelease()`, this object is put into the autorelease pool. The pool can help you to retain this objects lifecycle till the end of current frame. At the end of current frame, if this object hasnt been retained by any other class/container, it will be released automatically. For example, `layer->addChild(sprite)`, the sprite is added to the children list of the layer, its lifecycle is retained till the release of layer, but not the end of current frame.
Thats why you can not manage `Ref` life cycle in another new thread: at each end frame, autorelease object will be deleted, and you will meet with a crash when calling these deleted pointers in another thread.

Ref::release, retain and autorelease
-----------------

There are 4 functions of `Ref`, the meanings are

* release: add reference count by 1
* retain: minus reference count by 1
* autorelease: put the object in to `AutoReleasePool`, which means that cocos2d-x will invoke `release` automatically at the end of this frame
* getReferenceCount: as the meaning of the function, it will retain the reference count of this object

Using Static Constructor
------------------------

Most coco2d-x classes has a `create` static function. This function will:

* create a new instance
* do needed initialization
* invoke `autorelease()` if instance is created successfully
* return a pointer of the new instance

As you can see, the `create` method will invoke `autorelease` internally. Which means you don't have to invoke `release` if you don't invoke `retain` explicitly.