Take a Screenshot
========================

> version: Cocos2d-x v3.2-alpha0 and higher
> update: {{lastupdated_at}}

{{toc}}

`utils::captureScreen()` is added to save a screenshot. 

The declaration of `utils::captureScreen()` is 
```c++
void captureScreen(const std::function<void(bool, const std::string&)>& afterCaptured, const std::string& filename);
```
* afterCaptured
This function will be called after capturing commands are executed. The boolean return value indicates screenshot is captured successfully or not. The string return value is the path screenshot is saved in.
* filename
The name of screenshot. It can be just a filename, such as `screenshot.png`. It can also be a resolute path, such as `/sdcard/screenshot.png`.

The declaration of `utils::captureScreen()` for Lua is:
```lua
cc.utils:captureScreen(capturedHandler, filename)
```
* `capturedHandler` This function will be called after capturing commands are executed. The format of the captureHandler as follows:

```lua
local function capturedHandler(succeed, outputFile)
           --do something
end
```

 The `succeed` return value indicates screenshot is captured successfully or not. The string `outputFile` value is the path screenshot is saved in.
* filename
 The name of screenshot. It can be just a filename, such as `screenshot.png`. It can also be a resolute path, such as `/sdcard/screenshot.png`.

## Usage

```c++
void CaptureScreenTest::afterCaptured(bool succeed, const std::string& outputFile)
{
    if (succeed)
    {
        // show screenshot
        auto sp = Sprite::create(outputFile);
        addChild(sp, 0, childName);
        Size s = Director::getInstance()->getWinSize();
        sp->setPosition(s.width / 2, s.height / 2);
        sp->setScale(0.25);
    }
    else
    {
        log("Capture screen failed.");
    }
}

void CaptureScreenTest::capture()
{
    utils::captureScreen(CC_CALLBACK_2(CaptureScreenTest::afterCaptured, this),"CaptureScreenTest.png");
}
```

```lua
 local function afterCaptured(succeed, outputFile)
      if succeed then
           local sp = cc.Sprite:create(outputFile)
           layer:addChild(sp, 0, childTag)
           local winSize =  cc.Director:getInstance():getWinSize()
           sp:setPosition(winSize.width / 2, winSize.height / 2)
           sp:setScale(0.25)
      else
           cclog("Capture screen failed.")
      end
end

local function onCaptured(tag, sender)
      cc.Director:getInstance():getTextureCache():removeTextureForKey(fileName)
      layer:removeChildByTag(childTag)
      fileName = "CaptureScreenTest.png"
      cc.utils:captureScreen(afterCaptured, fileName)
end
```